#include <iostream.h>
#include <iomanip.h>
#include <fstream.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "lparx.h"
ifelse(NDIM,2,@
#include "cadez_hierarchy2.h"
#include "integrals2.h"
#include "symdata2.h"  // extern FORTRAN declarations for symdata2D.F
@,NDIM,3,@
#include "cadez_hierarchy3.h"
#include "integrals3.h"
#include "symdata3.h"  // extern FORTRAN declarations for symdata2D.F
@)
#include "cadezdefs.h"  // extern FORTRAN declarations for cadef.F & chicoord.F

//	LPARX implementations
//ifelse(NDIM,3,@
//implement_Grid3(double)
@)
//implement_Grid2(double)
//implement_Grid1(double)

void printX(XArray1(GridX(double)) & G);

void Make_Tau(GridX(double) &, const RegionX &,
					const GridX(double) &, const RegionX &);
GridReductionFunctionID Make_Tau_id =
		GridX(double)::RegisterReductionFunction(Make_Tau);
void CG_Correct(GridX(double) &, const RegionX &,
					const GridX(double) &, const RegionX &);
GridReductionFunctionID CG_Correct_id =
		GridX(double)::RegisterReductionFunction(CG_Correct);

// forward declairations of subroutines
ifelse(NDIM,2,@
void Coordinate_input(const int , char **, ifstream &,
		double&, double&, double&, int&, int&, int&, int&, int&, int&,
		double&, double&, int&, int&);
@,NDIM,3,@
void Coordinate_input(const int , char **, ifstream &,
		double&, double&, double&, int&, int&, int&, int&, int&, int&, int&,
		double&, double&, int&, int&);
@)

ifelse(NDIM,2,@
int Param_Set_input(ifstream &, double&, double&, double&, double&, double&);
@,NDIM,3,@
int Param_Set_input(ifstream &, double&, double&, double&, double&, double&,
		double&, double&, double&, double&, double&, double&, double&, double&);
@)

ifelse(NDIM,2,@
void Determine_coarse_grid(const double, const double, const double,
	const int, const int, int&, int&, int&, double&, double&, double&,
	double&, double&, double&, double&);
@,NDIM,3,@
void Determine_coarse_grid(const double, const double, const double,
	const int, const int, int&, int&, int&, int&, double&, double&, double&,
	double&, double&, double&, double&);
@)

double Psi_Smooth(cadez_discretization &, int);

int main(const int argc, char * argv[])
{
static TimeSection timer("OVERALL");
TimeSection::verbose(0);
timer.enter_section();
//
// Cadez coordinate variables
double Alpha = 1.0;			// Ratio of hole sizes.
double Beta = 10.0;			// Separation of holes.
double Radmino = 200.0;		// Minimum radius of outer boundary.
double ETAzp = 0.0;			// Cadez coordinate eta value on + hole
double ETAzm = 0.0;			// Cadez coordinate eta value on - hole
double ETAmax = 0.0;			// Cadez coordinate eta value on outer boundary
double XIi = 0.0;				// Cadez coordinate xi initial value
double XIf = 0.0;				// Cadez coordinate xi final value
double ZETAp = 5.0;			// Z coordinate of + hole
double ZETAm = -5.0;			// Z coordinate of - hole
double Zsing = 0.0;			// Z coordinate of Cadez singularity
double RHOsing = 0.0;		// Rho coordinate of Cadez singularity
double ETAsing = 0.0;		// Eta coordinate of Cadez singularity
double XIsing = 0.0;			// Xi coordinate of Cadez singularity
//
// Cadez coarse grid discretization variables
int Netapc = 3;				// Number of zones between + hole and singularity.
int Netamc = 3;				// Number of zones between - hole and singularity.
int Neta3c = 1;				// Number of zones between singularity & OB.
int Nspc = 1;					// Number of zones between + axis and singularity.
int Nsmc = 1;					// Number of zones between - axis and singularity.
int Ns = 2;						// Total number of angular zones.
ifelse(NDIM,3,@
int Nphic = 1;					// Total number of angular zones.
@)
//
int istop = 1;					// Just compute coordinate system info if non-zero.
//
// Extrinsic Curvature variables
double Isom = -1.0;
ifelse(NDIM,2,@
double Pzp = 0.0, Pzm = 0.0, Szp = 0.0, Szm = 0.0;
@,NDIM,3,@
double Pxp = 0.0, Pxm = 0.0, Sxp = 0.0, Sxm = 0.0;
double Pyp = 0.0, Pym = 0.0, Syp = 0.0, Sym = 0.0;
double Pzp = 0.0, Pzm = 0.0, Szp = 0.0, Szm = 0.0;
@)
//
// Multigrid variables
int MGlevs = 0;				// Minimum number of FULL multigrid levels.
int MGlevs_max = 0;			// Maximum number of ALL multigrid levels.
double Tau_acc = 0.0;		// Maximum allowed truncation error.
double Conv_frac = 1.e-2;	// Fraction of Tau which residual must achieve.
int MG_Verb = 0;				// Just compute coordinate system info if non-zero.
//
//	File variables
ifstream infile;

// Get the input defining the Cadez coordinate system and multigrid hierarchy.
//	Then Compute the desired discretization of the coarse grid.
ifelse(NDIM,2,@
	Coordinate_input(argc, argv, infile, Alpha, Beta, Radmino,
							Netapc, Netamc, Nspc, Nsmc,
							MGlevs, MGlevs_max, Tau_acc, Conv_frac, istop, MG_Verb);
	Determine_coarse_grid(Alpha, Beta, Radmino, Netapc, Netamc, Neta3c,
									Nspc, Nsmc, ETAzp, ETAzm, ETAmax, XIi, XIf,
									ZETAp, ZETAm);
@,NDIM,3,@
	Coordinate_input(argc, argv, infile, Alpha, Beta, Radmino,
							Netapc, Netamc, Nspc, Nsmc, Nphic,
							MGlevs, MGlevs_max, Tau_acc, Conv_frac, istop, MG_Verb);
	Determine_coarse_grid(Alpha, Beta, Radmino, Netapc, Netamc, Neta3c,
									Nspc, Nsmc, Nphic, ETAzp, ETAzm, ETAmax, XIi, XIf,
									ZETAp, ZETAm);
@)
	chising(ETAsing,XIsing);	// We need the location of the singularity to
	cylsing(Zsing,RHOsing);		// define the general Cadez information.

   initsym(Alpha,ZETAp,ZETAm);	// Initialize TS data routines.
	double TS_energy,TS_dipole;
	tsenergy(TS_energy);	// Compute the Time-Symmetric ADM energy.
	tsdipole(TS_dipole);	// Compute the Time-Symmetric dipole moment.
	if(mpMyID() == 0) {
		cout << "Analytic Time-Symmetric ADM energy    : " << TS_energy << endl;
		cout << "Analytic Time-Symmetric dipole moment : " << TS_dipole << endl;
	}
//
// Define the variable cadez as the Cadez hierarchy and set the general
// Cadez information.
	cadez_hierarchy cadez(Alpha,Beta,ZETAp,ZETAm,ETAzp,ETAzm,ETAmax,XIi,XIf,
								Zsing,RHOsing,ETAsing,XIsing);
//
   if(mpMyID() == 0) cout << cadez << endl;
//
// Set the discretization of the coarse grid.
ifelse(NDIM,2,@
	cadez.coarse_level(Netapc,Netamc,Neta3c,Nspc,Nsmc);
@,NDIM,3,@
	cadez.coarse_level(Netapc,Netamc,Neta3c,Nspc,Nsmc,Nphic);
@)
	if(mpMyID() == 0) cadez.printlevels(cout);
//
//	Stop here if istop = 1
	if(istop == 1) exit(0);
//
// Get the input defining the momenta of the holes.
ifelse(NDIM,2,@
	int valid_params = Param_Set_input(infile,Isom,Pzp,Pzm,Szp,Szm);
	cadez.SetMomenta(Pzp,Pzm,Szp,Szm,Isom);
@,NDIM,3,@
	int valid_params = Param_Set_input(infile,
							Isom,Pxp,Pxm,Pyp,Pym,Pzp,Pzm,Sxp,Sxm,Syp,Sym,Szp,Szm);
	cadez.SetMomenta(Pxp,Pxm,Pyp,Pym,Pzp,Pzm,
							Sxp,Sxm,Syp,Sym,Szp,Szm,Isom);
@)
//
// Compute Multigrid Solution
	while(valid_params) {
		int lev = 0;
		cadez.New_Parameter_Set(lev);
		do {
			int loop = 0;
			F_ID tau, res;
			do {
				++loop;
				if(mpMyID() == 0 && MG_Verb > 0) 
					cout << " Multigrid Iteration : " << loop << endl;
				cadez.mg_init_data(lev, res, tau, MG_Verb);
				if(mpMyID() == 0 && MG_Verb > 0) {
					const long flagsave = cout.setf(ios::scientific|ios::right);
					const int precsave = cout.precision(2);
					cout << "L2:Tau| " << setw(8) << tau.Psi << " |" << endl;
					cout << "L2:Res| " << setw(8) << res.Psi << " |" << endl;
					cout.flags(flagsave);
					cout.precision(precsave);
				}
			}
			while(res > tau*Conv_frac && loop < 10);
			if(mpMyID() == 0 && MG_Verb == 0) {
				const long flagsave = cout.setf(ios::scientific|ios::right);
				const int precsave = cout.precision(2);
				cout << "Multigrid Iterations : " << loop << endl;
				cout << "L2:Tau| " << setw(8) << tau.Psi << " |" << endl;
				cout << "L2:Res| " << setw(8) << res.Psi << " |" << endl;
				cout.flags(flagsave);
				cout.precision(precsave);
			}
			const double N_energy = ADM_energy(cadez);
ifelse(NDIM,3,@
			const double N_dip_xc = Dipole_X(cadez);
			const double N_dip_yc = Dipole_Y(cadez);
@)
			const double N_dip_zc = Dipole_Z(cadez);
			const double N_Area_p = MS_Area(cadez,1);
			const double N_Area_m = MS_Area(cadez,-1);
			const double N_Sep = MS_Separation(cadez);
  			if(mpMyID() == 0) {
				const long flagsave = cout.setf(ios::right);
				const int precsave = cout.precision(8);
				cout << "   ADM energy        : " <<setw(14) << N_energy << endl;
ifelse(NDIM,2,@
				cout << "   Dipole moment : z : " <<setw(14) << N_dip_zc << endl;
@,NDIM,3,@
				cout << "   Dipole moment : x : " <<setw(14) << N_dip_xc << endl;
				cout << "                   y : " <<setw(14) << N_dip_yc << endl;
				cout << "                   z : " <<setw(14) << N_dip_zc << endl;
@)
				cout << "   Area:      + hole : " <<setw(14) << N_Area_p << endl;
				cout << "              - hole : " <<setw(14) << N_Area_m << endl;
				cout << "   Proper Separation : " <<setw(14) << N_Sep << endl;
				cout.flags(flagsave);
				cout.precision(precsave);
			}
		//
		//	Refine solution
			if(lev < MGlevs) cadez.uniform_refine(++lev);
			else if(lev < MGlevs_max && cadez.adapt_refine(++lev,Tau_acc));
			else	break;
			if(mpMyID() == 0)
					cout << "Level : " << lev << '\n' << cadez[lev] << endl;
		}
		while(lev <= MGlevs_max);
		cadez.define_max_level(lev);
	//
	// Get another set of momenta parameters
ifelse(NDIM,2,@
		valid_params = Param_Set_input(infile,Isom,Pzp,Pzm,Szp,Szm);
		cadez.SetMomenta(Pzp,Pzm,Szp,Szm,Isom);
@,NDIM,3,@
		valid_params = Param_Set_input(infile,
						Isom,Pxp,Pxm,Pyp,Pym,Pzp,Pzm,Sxp,Sxm,Syp,Sym,Szp,Szm);
		cadez.SetMomenta(Pxp,Pxm,Pyp,Pym,Pzp,Pzm,
								Sxp,Sxm,Syp,Sym,Szp,Szm,Isom);
@)
	}
//
//	Close input file when done with it
	infile.close();
//
//	Timing Information
	double time = mpCPUSeconds();
	timer.exit_section();
	double totaltime = time;
	double maxtime = time;
	mpReduceMax(&maxtime);
	mpReduceAdd(&totaltime);
	int myID = mpMyID();
   if(myID > 0) mpSend(myID,&time,sizeof(time),0);
	if(mpMyID() == 0) {
		int nNodes = mpNodes();
		cout << "------------------------------------------------" << endl;
		cout << "| Processor " << setw(3) << myID << " CPU time : " 
									<< setw(10) << time << " seconds. |" << endl;
		for(int in = 1; in < nNodes ; ++in) {
			mpRecv(in,&time,sizeof(time));
			cout << "| Processor " << setw(3) << in << " CPU time : " 
									<< setw(10) << time << " seconds. |" << endl;
		}
		cout << "------------------------------------------------" << endl;
		cout << "|       Maximum CPU time : "
								<< setw(10) << maxtime << " seconds. |" << endl;
		cout << "|         Total CPU time : "
								<< setw(10) << totaltime << " seconds. |" << endl;
		cout << "------------------------------------------------" << endl;
	}
	TimeSection::report_times();

// Exit properly
	exit(0);
} // main

//------------------------------------------------------------------------------
// Read in the Cadez coordinate system and multigrid information
//------------------------------------------------------------------------------
ifelse(NDIM,2,@
void Coordinate_input(const int Nargs, char * argv[], ifstream & infile,
	double& Alpha, double& Beta, double& Radmino, int& Netapc, int& Netamc,
	int& Nspc, int& Nsmc, int& MGlevs, int& MGlevs_max,
	double& Tau_acc, double& Conv_frac, int& istop, int& MG_Verb) {
@,NDIM,3,@
void Coordinate_input(const int Nargs, char * argv[], ifstream & infile,
	double& Alpha, double& Beta, double& Radmino, int& Netapc, int& Netamc,
	int& Nspc, int& Nsmc, int& Nphic, int& MGlevs, int& MGlevs_max,
	double& Tau_acc, double& Conv_frac, int& istop, int& MG_Verb) {
@)
	const int realsiz = sizeof(Alpha);
	const int intsiz = sizeof(Netapc);
	if(mpMyID() == 0) {
		if(Nargs == 1) {	// if no comand line args. read input from keyboard.
			cout << "Enter Cadez coordinate definitions." << endl;
			cout << "Alpha = ";
			cin >> Alpha;
			cout << "Beta = ";
			cin >> Beta;
			cout << "Min. outer radius = ";
			cin >> Radmino;
			cout << "Enter Cadez discretization definitions." << endl;
			cout << "      Radial disc." << endl;
			cout << "             N_eta+ = ";
			cin >> Netapc;
			cout << "             N_eta- = ";
			cin >> Netamc;
			cout << "      Angular disc." << endl;
			cout << "             N_s+ = ";
			cin >> Nspc;
			cout << "             N_s- = ";
			cin >> Nsmc;
ifelse(NDIM,3,@
			cout << "      Phi disc." << endl;
			cout << "             N_phi = ";
			cin >> Nphic;
@)
			cout << "\n=============================================\n " << endl;
		} else if(Nargs == 2) {	// 1 argument, read input from specified file.
			infile.open(argv[1]);
			if(!infile){
				cerr << "Can't open " << argv[1] << " for input." << endl;
				exit(1);
			}
			char buf[100],c;
			infile.get(buf,100,':'); infile.get(c); infile >> Alpha;
			infile.get(buf,100,':'); infile.get(c); infile >> Beta;
			infile.get(buf,100,':'); infile.get(c); infile >> Radmino;
			infile.get(buf,100,':'); infile.get(c); infile >> Netapc;
			infile.get(buf,100,':'); infile.get(c); infile >> Netamc;
			infile.get(buf,100,':'); infile.get(c); infile >> Nspc;
			infile.get(buf,100,':'); infile.get(c); infile >> Nsmc;
ifelse(NDIM,3,@
			infile.get(buf,100,':'); infile.get(c); infile >> Nphic;
@)
			infile.get(buf,100,':'); infile.get(c); infile >> MGlevs;
			infile.get(buf,100,':'); infile.get(c); infile >> MGlevs_max;
			infile.get(buf,100,':'); infile.get(c); infile >> Tau_acc;
			infile.get(buf,100,':'); infile.get(c); infile >> Conv_frac;
			infile.get(buf,100,':'); infile.get(c); infile >> istop;
			infile.get(buf,100,':'); infile.get(c); infile >> MG_Verb;
			infile.getline(buf,100);	// "---..."
			infile.getline(buf,100);	// "Extrinsic Curvature"
			if(infile.eof()) {
				cerr << "Error reading input file " << argv[1] << endl;
				exit(1);
			}
		} else {	// Incorrect command syntax.  Que the user.
			cerr << "Syntax:  cadez [input_file]" << endl;
			exit(2);
		}
	//
		MGlevs = (MGlevs > 0) ? MGlevs : 0;
		MGlevs_max = (MGlevs_max > 0) ? MGlevs_max : 0;
		MGlevs = (MGlevs <= MGlevs_max) ? MGlevs : MGlevs_max;
	//
	// echo out input data
		cout << "            alpha : " << Alpha << endl;
		cout << "             beta : " << Beta << endl;
		cout << "Min. outer radius : " << Radmino << endl;
		cout << "    radial> neta+ : " << Netapc << endl;
		cout << "            neta- : " << Netamc << endl;
		cout << "     angular> ns+ : " << Nspc << endl;
		cout << "              ns- : " << Nsmc << endl;
ifelse(NDIM,3,@
		cout << "    azimuth> nphi : " << Nphic << endl;
@)
		cout << "  multigrid depth : " << MGlevs << endl;
		cout << "    multigrid max : " << MGlevs_max << endl;
		cout << " Max. Trunc. Err. : " << Tau_acc << endl;
		cout << "Convergence Frac. : " << Conv_frac << endl;
		cout << "      Header Only : " << istop << endl;
		cout << "     MG-Verposity : " << MG_Verb << endl;
		cout << "--------------------------------------------------------"
					"----------------" << endl;
		cout << "Extrinsic Curvature" << endl;
		//
		mpBroadcast(&Alpha,realsiz,0);
		mpBroadcast(&Beta,realsiz,0);
		mpBroadcast(&Radmino,realsiz,0);
		mpBroadcast(&Netapc,intsiz,0);
		mpBroadcast(&Netamc,intsiz,0);
		mpBroadcast(&Nspc,intsiz,0);
		mpBroadcast(&Nsmc,intsiz,0);
ifelse(NDIM,3,@
		mpBroadcast(&Nphic,intsiz,0);
@)
		mpBroadcast(&MGlevs,intsiz,0);
		mpBroadcast(&MGlevs_max,intsiz,0);
		mpBroadcast(&Tau_acc,realsiz,0);
		mpBroadcast(&Conv_frac,realsiz,0);
		mpBroadcast(&istop,intsiz,0);
		mpBroadcast(&MG_Verb,intsiz,0);
	} else {
		mpBroadcast(&Alpha,realsiz,0);
		mpBroadcast(&Beta,realsiz,0);
		mpBroadcast(&Radmino,realsiz,0);
		mpBroadcast(&Netapc,intsiz,0);
		mpBroadcast(&Netamc,intsiz,0);
		mpBroadcast(&Nspc,intsiz,0);
		mpBroadcast(&Nsmc,intsiz,0);
ifelse(NDIM,3,@
		mpBroadcast(&Nphic,intsiz,0);
@)
		mpBroadcast(&MGlevs,intsiz,0);
		mpBroadcast(&MGlevs_max,intsiz,0);
		mpBroadcast(&Tau_acc,realsiz,0);
		mpBroadcast(&Conv_frac,realsiz,0);
		mpBroadcast(&istop,intsiz,0);
		mpBroadcast(&MG_Verb,intsiz,0);
	}
} // Coordinate_input


//------------------------------------------------------------------------------
// Read in the isometry condition and momenta for the holes.
//------------------------------------------------------------------------------
ifelse(NDIM,2,@
int Param_Set_input(ifstream & infile, double& Isom,
	double& Pzp, double& Pzm, double& Szp, double& Szm) {
@,NDIM,3,@
int Param_Set_input(ifstream & infile, double& Isom,
	double& Pxp, double& Pxm, double& Pyp, double& Pym,
	double& Pzp, double& Pzm, double& Sxp, double& Sxm,
	double& Syp, double& Sym, double& Szp, double& Szm) {
@)
	const int realsiz = sizeof(Isom);
	if(mpMyID() == 0) {
		if(!infile){
			cerr << "Parameter set input file not open for input." << endl;
			exit(1);
		}
		char buf[100],c;
		infile.get(buf,100,':'); infile.get(c); infile >> Isom;
ifelse(NDIM,2,@
		infile.get(buf,100,':'); infile.get(c);
		infile.get(buf,100,','); infile.get(c);
		infile.get(buf,100,','); infile.get(c); infile >> Pzp;
		infile.get(buf,100,':'); infile.get(c);
		infile.get(buf,100,','); infile.get(c);
		infile.get(buf,100,','); infile.get(c); infile >> Pzm;
		infile.get(buf,100,':'); infile.get(c);
		infile.get(buf,100,','); infile.get(c);
		infile.get(buf,100,','); infile.get(c); infile >> Szp;
		infile.get(buf,100,':'); infile.get(c);
		infile.get(buf,100,','); infile.get(c);
		infile.get(buf,100,','); infile.get(c); infile >> Szm;
@,NDIM,3,@
		infile.get(buf,100,':'); infile.get(c); infile >> Pxp;
		infile.get(buf,100,','); infile.get(c); infile >> Pyp;
		infile.get(buf,100,','); infile.get(c); infile >> Pzp;
		infile.get(buf,100,':'); infile.get(c); infile >> Pxm;
		infile.get(buf,100,','); infile.get(c); infile >> Pym;
		infile.get(buf,100,','); infile.get(c); infile >> Pzm;
		infile.get(buf,100,':'); infile.get(c); infile >> Sxp;
		infile.get(buf,100,','); infile.get(c); infile >> Syp;
		infile.get(buf,100,','); infile.get(c); infile >> Szp;
		infile.get(buf,100,':'); infile.get(c); infile >> Sxm;
		infile.get(buf,100,','); infile.get(c); infile >> Sym;
		infile.get(buf,100,','); infile.get(c); infile >> Szm;
@)
		if(infile.eof()) return 0;
		mpBroadcast(&Isom,realsiz,0);
ifelse(NDIM,3,@
		mpBroadcast(&Pxp,realsiz,0);
		mpBroadcast(&Pxm,realsiz,0);
		mpBroadcast(&Sxp,realsiz,0);
		mpBroadcast(&Sxm,realsiz,0);
		mpBroadcast(&Pyp,realsiz,0);
		mpBroadcast(&Pym,realsiz,0);
		mpBroadcast(&Syp,realsiz,0);
		mpBroadcast(&Sym,realsiz,0);
@)
		mpBroadcast(&Pzp,realsiz,0);
		mpBroadcast(&Pzm,realsiz,0);
		mpBroadcast(&Szp,realsiz,0);
		mpBroadcast(&Szm,realsiz,0);
		if(Isom != 1.0 && Isom != -1.0) return 0;
	//
	//	echo out input data
		cout << "\nisometry (+1/-1) : " << Isom << endl;
ifelse(NDIM,2,@
		cout << "       P+[x,y,z] : " << setw(14) << 0.0 << " , "
				<< setw(14) << 0.0 << " , " << setw(14) << Pzp << endl;
		cout << "       P-[x,y,z] : " << setw(14) << 0.0 << " , "
				<< setw(14) << 0.0 << " , " << setw(14) << Pzm << endl;
		cout << "       S+[x,y,z] : " << setw(14) << 0.0 << " , "
				<< setw(14) << 0.0 << " , " << setw(14) << Szp << endl;
		cout << "       S-[x,y,z] : " << setw(14) << 0.0 << " , "
				<< setw(14) << 0.0 << " , " << setw(14) << Szm << endl;
@,NDIM,3,@
		cout << "       P+[x,y,z] : " << setw(14) << Pxp << " , "
				<< setw(14) << Pyp << " , " << setw(14) << Pzp << endl;
		cout << "       P-[x,y,z] : " << setw(14) << Pxm << " , "
				<< setw(14) << Pym << " , " << setw(14) << Pzm << endl;
		cout << "       S+[x,y,z] : " << setw(14) << Sxp << " , "
				<< setw(14) << Syp << " , " << setw(14) << Szp << endl;
		cout << "       S-[x,y,z] : " << setw(14) << Sxm << " , "
				<< setw(14) << Sym << " , " << setw(14) << Szm << endl;
@)
	} else {
		mpBroadcast(&Isom,realsiz,0);
ifelse(NDIM,3,@
		mpBroadcast(&Pxp,realsiz,0);
		mpBroadcast(&Pxm,realsiz,0);
		mpBroadcast(&Sxp,realsiz,0);
		mpBroadcast(&Sxm,realsiz,0);
		mpBroadcast(&Pyp,realsiz,0);
		mpBroadcast(&Pym,realsiz,0);
		mpBroadcast(&Syp,realsiz,0);
		mpBroadcast(&Sym,realsiz,0);
@)
		mpBroadcast(&Pzp,realsiz,0);
		mpBroadcast(&Pzm,realsiz,0);
		mpBroadcast(&Szp,realsiz,0);
		mpBroadcast(&Szm,realsiz,0);
		if(Isom != 1.0 && Isom != -1.0) return 0;
	}
	return 1;
} // Param_Set_input


//------------------------------------------------------------------------------
// Determine the coarse grid Cadez coordinate discretization.
//------------------------------------------------------------------------------
ifelse(NDIM,2,@
void Determine_coarse_grid(const double Alpha, const double Beta,
	const double Radmino, const int Netapc, const int Netamc, int& Neta3c,
	int& Nspc, int& Nsmc, double& ETAzp, double& ETAzm, double& ETAmax,
	double& XIi, double& XIf, double& ZETAp, double& ZETAm) {
@,NDIM,3,@
void Determine_coarse_grid(const double Alpha, const double Beta,
	const double Radmino, const int Netapc, const int Netamc, int& Neta3c,
	int& Nspc, int& Nsmc, int& Nphic, double& ETAzp, double& ETAzm,
	double& ETAmax, double& XIi, double& XIf, double& ZETAp, double& ZETAm) {
@)
static TimeSection timer("Determine_coarse_grid()");
timer.enter_section();
double Sigma = 0.0;			// Bispherical sigma coefficient (ignore this)
double Zsing = 0.0;			// Z coordinate of Cadez singularity
double RHOsing = 0.0;		// Rho coordinate of Cadez singularity
double ETAsing = 0.0;		// Eta coordinate of Cadez singularity
double XIsing = 0.0;		// Xi coordinate of Cadez singularity
double eta = 0.0, xi = 0.0;
int nsi;

// Call FORTRAN routines to define the Cadez coordinate system for 
// specified Alpha & Beta (along with Netapc & Netamc).
	cadef(Alpha,Beta,Netapc,Netamc,Sigma,ETAzp,ETAzm,ZETAp,ZETAm);
	chising(ETAsing,XIsing);
	cylsing(Zsing,RHOsing);

// calculate the coarse eta coordinate spacing deta
	const double detac = (ETAsing - ETAzp)/Netapc;

// determine outer eta coordinate which is at or beyond radmino
// and determine number of grid points needed to reach radmino
	cyl2chi(Zsing,Radmino,eta,xi);
	Neta3c = (eta - ETAsing + 0.5*detac)/detac + 1;

// make Neta3c a small prime times a power of 2
	int n3a, ne3a = -1;
	do
		n3a = pow(2,++ne3a);
	while(n3a < Neta3c);
	int n3b, ne3b = -1;
	do
		n3b = 3*pow(2,++ne3b);
	while(n3b < Neta3c);
	int n3c, ne3c = -1;
	do
		n3c = 5*pow(2,++ne3c);
	while(n3c < Neta3c);
	Neta3c = (n3a < n3b) ? n3a : n3b;
	Neta3c = (Neta3c < n3c) ? Neta3c : n3c;
	if(Neta3c == n3a)			nsi= (ne3a-3 > 0) ? ne3a-3 : 0;
	else if(Neta3c == n3b)	nsi= (ne3b-2 > 0) ? ne3b-2 : 0;
	else							nsi= (ne3c-1 > 0) ? ne3c-1 : 0;

// compute maximum eta coordinate value
	ETAmax = Neta3c*detac + ETAsing;

// compute coarse grid xi discretization
	Nspc = ((Nspc *= pow(2,nsi)) > 4) ? Nspc : 4;
	Nsmc = ((Nsmc *= pow(2,nsi)) > 4) ? Nsmc : 4;

// compute xi coordinate bounds
	XIi = 0.0;
	XIf = M_PI*coefnorm();

ifelse(NDIM,3,@
// compute coarse grid phi discretization
	Nphic = ((Nphic *= pow(2,nsi)) > 4) ? Nphic : 4;
@)
timer.exit_section();
} // Determine_coarse_grid
