c#######################################################################
c#
c#                Cadez Coordinate Initialization
c#
c#######################################################################
      function cadez_coord_init(Xi,Eta,X,Y,JXzc,JXzf,JYzc,JYzf,
     .                            RHOzc,RHOxf,RHOyf,Zzc,Zxf,Zyf,
     .                            Volume,CJACzc,CJACxf,CJACyf,
     .                            ext,fil,
     .                            ds,deta,f,g,XIsing,ETAsing)
      implicit none
c-----------------------------------------------------------------------
      integer cadez_coord_init,ext(4),fil(4)
      Real*8 Xi(ext(1):ext(3)),X(ext(1):ext(3))
      Real*8 JXzc(ext(1):ext(3)),JXzf(ext(1):ext(3))
      Real*8 Eta(ext(2):ext(4)),Y(ext(2):ext(4))
      Real*8 JYzc(ext(2):ext(4)),JYzf(ext(2):ext(4))
      Real*8 RHOzc(ext(1):ext(3),ext(2):ext(4))
      Real*8 RHOxf(ext(1):ext(3),ext(2):ext(4))
      Real*8 RHOyf(ext(1):ext(3),ext(2):ext(4))
      Real*8 Zzc(ext(1):ext(3),ext(2):ext(4))
      Real*8 Zxf(ext(1):ext(3),ext(2):ext(4))
      Real*8 Zyf(ext(1):ext(3),ext(2):ext(4))
      Real*8 Volume(ext(1):ext(3),ext(2):ext(4))
      Real*8 CJACzc(ext(1):ext(3),ext(2):ext(4))
      Real*8 CJACxf(ext(1):ext(3),ext(2):ext(4))
      Real*8 CJACyf(ext(1):ext(3),ext(2):ext(4))
      Real*8 ds,deta,f,g,XIsing,ETAsing
c-----------------------------------------------------------------------
      integer i,j,region,err,chi2cyl
      integer iminf,imaxf,jminf,jmaxf
      integer imine,imaxe,jmine,jmaxe
      Real*8 Etac,Xic,rho,z,dedr,dedz
      Real*8 ZERO,HALF,ONE,TWO,PI,TWOPI,CADACC
      parameter(ZERO = 0.D0,HALF = 0.5D0,ONE = 1.D0,TWO = 2.D0)
      parameter(PI = 3.1415926535897931160D0, TWOPI = 2.D0*PI)
      parameter(CADACC = 1.D-10)
c
c-----------------------------------------------------------------------
      cadez_coord_init = 0
c
c-----------------------------------------------------------------------
      iminf = fil(1)
      imaxf = fil(3)
      jminf = fil(2)
      jmaxf = fil(4)
      imine = ext(1)
      imaxe = ext(3)
      jmine = ext(2)
      jmaxe = ext(4)
c
c-----------------------------------------------------------------------
c- Set the (X,Y) and (Xi,Eta) coordinates on Zone Centers
c-----------------------------------------------------------------------
      do i = imine,imaxe
        X(i) = ds*(i - HALF)
        Xi(i) = X(i)*(f*X(i) + g)
      end do
      do j = jmine,jmaxe
        Y(j) = deta*(j - HALF) + ETAsing
        Eta(j) = Y(j)
      end do
c
c-----------------------------------------------------------------------
c- Set the X/Y coordinate Jacobians on Zone Centers
c-----------------------------------------------------------------------
      do i = iminf,imaxf
        JXzc(i) = ONE/(ds*(TWO*f*X(i) + g))
      end do
      do j = jminf,jmaxe
        JYzc(j) = ONE/deta
      end do
c
c-----------------------------------------------------------------------
c- Set rho and z on Zone Centers
c- Set the Conformal Coordinate Jacobian on Zone Centers
c-----------------------------------------------------------------------
      do i = iminf,imaxf
        Xic = Xi(i)
        if(Xic.lt.XIsing) then
          region = 0
        else
          region = 1
        end if
        do j = jminf,jmaxe
          err = chi2cyl(Eta(j),Xic,z,rho,CADACC,region)
          if(err.eq.1) then
            write(*,*) "1 :",i,j,Eta(j),Xic,region
            cadez_coord_init = 1
            return
          end if
          call diffeta1(z,rho,dedr,dedz)
          Zzc(i,j) = z
          RHOzc(i,j) = rho
          CJACzc(i,j) = dedr**2 + dedz**2
          Volume(i,j) = TWOPI*RHOzc(i,j)/(CJACzc(i,j)*JXzc(i)*JYzc(j))
        end do
      end do
c
c-----------------------------------------------------------------------
c- Set the X/Y coordinate Jacobians on Zone Faces
c-----------------------------------------------------------------------
      do i = imine,imaxe-1
        JXzf(i) = ONE/(ds*(TWO*ds*i*f + g))
      end do
      do j = jmine,jmaxe-1
        JYzf(j) = ONE/deta
      end do
c
c-----------------------------------------------------------------------
c- Set rho on X-Zone Faces
c- Set the Conformal Coordinate Jacobian on X-Zone Faces
c-----------------------------------------------------------------------
      do i = imine,imaxe-1
        Xic = ds*i
        Xic = Xic*(f*Xic + g)
        if(Xic.eq.XIsing) then
          if(Xi(iminf).lt.XIsing) then
            region = 0
          else
            region = 1
          end if
        else if(Xic.lt.XIsing) then
          region = 0
        else
          region = 1
        end if
        do j = jminf,jmaxe
          err = chi2cyl(Eta(j),Xic,z,rho,CADACC,region)
          if(err.eq.1) then
            write(*,*) "2 :",i,j,Eta(j),Xic,region
            cadez_coord_init = 1
            return
          end if
          call diffeta1(z,rho,dedr,dedz)
          Zxf(i,j) = z
          RHOxf(i,j) = rho
          CJACxf(i,j) = dedr**2 + dedz**2
        end do
      end do
c
c-----------------------------------------------------------------------
c- Set rho on Y-Zone Faces
c- Set the Conformal Coordinate Jacobian on Y-Zone Faces
c-----------------------------------------------------------------------
      do i = iminf,imaxf
        Xic = Xi(i)
        if(Xic.lt.XIsing) then
          region = 0
        else
          region = 1
        end if
        do j = jmine,jmaxe-1
          Etac = deta*j + ETAsing
          err = chi2cyl(Etac,Xic,z,rho,CADACC,region)
          if(err.eq.1) then
            write(*,*) "3 :",i,j,Etac,Xic,region
            cadez_coord_init = 1
            return
          end if
          call diffeta1(z,rho,dedr,dedz)
          Zyf(i,j) = z
          RHOyf(i,j) = rho
          CJACyf(i,j) = dedr**2 + dedz**2
        end do
      end do
c
      return
      end
