//: cadez_hierarchy.h-- Class definitions for objects to describe a hierarchy
//. of grids for the Cadez coordinate system
#ifndef CADEZ_HIERARCHY_H_
#define CADEZ_HIERARCHY_H_

// LPARX declarations
ifelse(NDIM,3,@
//declare_Grid3(double)
#include "XArray1Grid3double.h"
@)
//declare_Grid2(double)
//declare_Grid1(double)
#include "XArray1Grid2double.h"
#include "XArray1Grid1double.h"

struct F_ID {
	double Psi;
	F_ID(double init = 0.0) {Psi = init;}
	F_ID operator*(double);
	int operator<(F_ID);
	int operator>(F_ID);
	int operator==(double);
};

class cadez_domain {
public:
	double Alpha;			// Ratio of hole sizes.
	double Beta;			// Separation of holes.
	double ZETAp;			// Z coordinate of "+" hole
	double ZETAm;			// Z coordinate of "-" hole
	double ETAzp;			// Cadez coordinate eta value on "+" hole
	double ETAzm;			// Cadez coordinate eta value on "-" hole
	double ETAmax;			// Cadez coordinate eta value on outer boundary
	double XIi;				// Cadez coordinate xi initial value
	double XIf;				// Cadez coordinate xi final value
	double Zsing;			// Z coordinate of Cadez singularity
	double RHOsing;		// Rho coordinate of Cadez singularity
	double ETAsing;		// Eta coordinate of Cadez singularity
	double XIsing;			// Xi coordinate of Cadez singularity
//
ifelse(NDIM,2,@
	double Pzp;				// Mag. of Z comp. of linear momentum on + hole
	double Pzm;				// Mag. of Z comp. of linear momentum on - hole
	double Szp;				// Mag. of Z comp. of angular momentum on + hole
	double Szm;				// Mag. of Z comp. of angular momentum on - hole
@,NDIM,3,@
	double Pxp;				// Mag. of X comp. of linear momentum on + hole
	double Pyp;				// Mag. of Y comp. of linear momentum on + hole
	double Pzp;				// Mag. of Z comp. of linear momentum on + hole
	double Pxm;				// Mag. of X comp. of linear momentum on - hole
	double Pym;				// Mag. of Y comp. of linear momentum on - hole
	double Pzm;				// Mag. of Z comp. of linear momentum on - hole
	double Sxp;				// Mag. of X comp. of angular momentum on + hole
	double Syp;				// Mag. of Y comp. of angular momentum on + hole
	double Szp;				// Mag. of Z comp. of angular momentum on + hole
	double Sxm;				// Mag. of X comp. of angular momentum on - hole
	double Sym;				// Mag. of Y comp. of angular momentum on - hole
	double Szm;				// Mag. of Z comp. of angular momentum on - hole
@)
	double Isom;			// Isometry condition (+1/-1)
//
	cadez_domain(double A, double B, double Zp, double Zm, double Ep,
		double Em, double Emax, double Xi, double Xf,
		double Zs, double Rs, double Es, double Xs) :
			Alpha(A), Beta(B), ZETAp(Zp), ZETAm(Zm), ETAzp(Ep),
			ETAzm(Em), ETAmax(Emax), XIi(Xi), XIf(Xf), Zsing(Zs),
			RHOsing(Rs), ETAsing(Es), XIsing(Xs) {
ifelse(NDIM,2,@
				Pzp = 0.0;
				Pzm = 0.0;
				Szp = 0.0;
				Szm = 0.0;
@,NDIM,3,@
				Pxp = 0.0;
				Pyp = 0.0;
				Pzp = 0.0;
				Pxm = 0.0;
				Pym = 0.0;
				Pzm = 0.0;
				Sxp = 0.0;
				Syp = 0.0;
				Szp = 0.0;
				Sxm = 0.0;
				Sym = 0.0;
				Szm = 0.0;
@)
				Isom = -1.0;
	}
	friend ostream& operator<<(ostream& s, cadez_domain& c);
};

//	Types of gridding
enum Grid_Type	{None = 0, Uniform = 1, Adapted = 2};

class cadez_discretization {
// General Cadez coordinate system information.
	cadez_domain& cdomain;
//	Type of gridding being used
	Grid_Type Grid_Kind;
public:
// Basic Cadez domain description.
//		The first coordinate is the Cadez \xi coordinate.
//		The second coordinate is the Cadez \eta coordinate.
	RegionX domain_R1; 	// Region 1 surrounds "+" hole.
								// Routines assume max-eta at region index of (*,0,*).
	RegionX domain_R2;	// Region 2 surrounds "-" hole.
								// Routines assume max-eta at region index of (*,0,*).
	RegionX domain_R3;	// Region 3 surrounds both holes.
								// Routines assume min-eta at region index of (*,1,*).
//
// Coordinate discretization info.
//		Note: \xi is considered a function of "s" via  \xi = s*(f*s + g).
	double deta;		// \eta discretization length.
	double ds;			// s discretization length.
ifelse(NDIM,3,@
	double dphi;		// phi discretization length.
@)
	double StoXI_f;	// f coefficient in \xi(s).
	double StoXI_g;	// g coefficient in \xi(s).
//
// Processor base IDs for the 3 Cadez regions
	int baseP_R1;			// Processor base for Cadez Region 1
	int baseP_R2;			// Processor base for Cadez Region 2
	int baseP_R3;			// Processor base for Cadez Region 3
//
// List of regions describing partitioning of the domain.
	int Ninteriors;		// Total number of partitions.
	RegionX *interiors;	// Computational interiors of each partition.
	RegionX *extents;		// Extent of the allocated grid for each partition.
	RegionX *fills;		// Part of each partition which should contain values.
//	A corresponding coarse grid exists to fascilitate inter-level interpolation.
	RegionX *coarse_interiors;
	RegionX *coarse_extents;
	RegionX *coarse_fills;
	RegionX *coarse_prolong;	// Interior for coarse to fine prolongation
// Processor mapping of each partition.
	int *mapping;
//
// Fill region duplicate point arrays.
	int dup_cnt;
	PointX *dup_pts;
	double *dup_value;
	double *dup_norm;
//------------------------------------------------------------------------------
//	Refinement mask.
	XArray1(GridX(double)) Refine_Mask;
//------------------------------------------------------------------------------
// Basic Coordinate information.
	XArray1(Grid1(double)) Xi;		// Conformal coordinate Xi
	XArray1(Grid1(double)) Eta;	// Conformal coordinate Eta
ifelse(NDIM,3,@
	XArray1(Grid1(double)) Phi;	// Conformal coordinate Phi
@)
	XArray1(Grid1(double)) X;		// Conformal coord BASE coordinate X
	XArray1(Grid1(double)) Y;		// Conformal coord BASE coordinate Y
ifelse(NDIM,3,@
	XArray1(Grid1(double)) W;		// Conformal coord BASE coordinate W
@)
	XArray1(Grid1(double)) JXzc;	// Conformal coord X-Jacobian : zone-centered
	XArray1(Grid1(double)) JXzf;	// Conformal coord X-Jacobian : zone-faced
	XArray1(Grid1(double)) JYzc;	// Conformal coord Y-Jacobian : zone-centered
	XArray1(Grid1(double)) JYzf;	// Conformal coord Y-Jacobian : zone-faced
ifelse(NDIM,3,@
	XArray1(Grid1(double)) JWzc;	// Conformal coord W-Jacobian : zone-centered
	XArray1(Grid1(double)) JWzf;	// Conformal coord W-Jacobian : zone-faced
@)
	XArray1(Grid2(double)) RHOzc;	// Cylindrical RHO coord : zone-centered
	XArray1(Grid2(double)) RHOxf;	// Cylindrical RHO coord : X-zone-faced
	XArray1(Grid2(double)) RHOyf;	// Cylindrical RHO coord : Y-zone-faced
	XArray1(Grid2(double)) Zzc;	// Cylindrical   Z coord : zone-centered
	XArray1(Grid2(double)) Zxf;	// Cylindrical   Z coord : X-zone-faced
	XArray1(Grid2(double)) Zyf;	// Cylindrical   Z coord : Y-zone-faced
	XArray1(Grid2(double)) Volume;// Conformal coord volume element
	XArray1(Grid2(double)) CJACzc;// Conformal coord Jacobian : zone-centered
	XArray1(Grid2(double)) CJACxf;// Conformal coord Jacobian : X-zone-faced
	XArray1(Grid2(double)) CJACyf;// Conformal coord Jacobian : Y-zone-faced
//------------------------------------------------------------------------------
// Differencing Coefficients
	XArray1(Grid2(double)) LapX;	// X-zone-faced Laplacian differencing
	XArray1(Grid2(double)) LapY;	// Y-zone-faced Laplacian differencing
ifelse(NDIM,3,@
	XArray1(Grid2(double)) LapW;	// W-zone-faced Laplacian differencing
@)
//------------------------------------------------------------------------------
//	Temporary Variables.
	XArray1(GridX(double)) TmpC; 		// Coarse Grid Temporary.
//------------------------------------------------------------------------------
// Physical Fields.
	XArray1(GridX(double)) Psi; 		// Conformal Factor
	XArray1(GridX(double)) PsiD; 		// Conformal Factor - 1
	XArray1(GridX(double)) PsiTau; 	// Conformal Factor Truncation Error.
	XArray1(GridX(double)) PsiRes; 	// Conformal Factor Residual
	XArray1(GridX(double)) PsiC; 		// Coarse Grid Conformal Factor
	XArray1(GridX(double)) PsiCT;		// Coarse Grid Conformal Factor
	XArray1(GridX(double)) KK; 		// Background extrinsic curvature squared
//------------------------------------------------------------------------------
//
ifelse(NDIM,2,@
	cadez_discretization(cadez_domain& cd, Grid_Type gt,
								int nep, int nem, int ne3, int nsp, int nsm);
@,NDIM,3,@
	cadez_discretization(cadez_domain& cd, Grid_Type gt,
								int nep, int nem, int ne3, int nsp, int nsm, int nphi);
@)
	~cadez_discretization();
	int Is_Uniform() { return Grid_Kind == Uniform;};
	int Is_Adapted() { return Grid_Kind == Adapted;};
	int domain_partition(const int P, const PointX minsize,
								int coarsen=1, int baseP = 0, int BSlevels = 0);
	void fill_patch(XArray1(GridX(double))&);
	void sync_fields();
	friend ostream& operator<<(ostream& s, cadez_discretization& c);
};

class cadez_hierarchy {
	enum {resize_chunk = 10};
//
// General Cadez coordinate system information (used by each level).
	cadez_domain domain;
//
// dynamically resizable storage for multigrid levels.
	cadez_discretization ** level;		// Multigrid levels (full levels).
	int size;									// Size of the level array.
	int nlevels;								// Number of levels being used.
//
// storage for Block-multigrid solver for Region 3.
	cadez_discretization ** BS_level;	// Multgrid levels (for Region 3 only).
	int BS_nlevels;							// Number of "Region 3" levels.
//
	double *Mat_R1;
	double *Mat_R2;
	double *Mat_R3;
	double *Upd_R1;
	double *Upd_R2;
	double *Upd_R3;
	void mg3(const int, F_ID&, F_ID&, const int pr = 0);
public:
	cadez_hierarchy(double A, double B, double Zp, double Zm, double Ep,
		double Em, double Emax, double Xi, double Xf,
		double Zs, double Rs, double Es, double Xs) :
			domain(A,B,Zp,Zm,Ep,Em,Emax,Xi,Xf,Zs,Rs,Es,Xs) {
		level = 0;
		size = 0;
		nlevels = 0;
		BS_level = 0;
		BS_nlevels = 0;
		Mat_R1 = 0;
		Mat_R2 = 0;
		Mat_R3 = 0;
		Upd_R1 = 0;
		Upd_R2 = 0;
		Upd_R3 = 0;
	}
	~cadez_hierarchy();
	double Alpha() {return domain.Alpha;}
	double Beta() {return domain.Beta;}
	double ZETAp() {return domain.ZETAp;}
	double ZETAm() {return domain.ZETAm;}
	double ETAzp() {return domain.ETAzp;}
	double ETAzm() {return domain.ETAzm;}
	double ETAmax() {return domain.ETAmax;}
	double XIi() {return domain.XIi;}
	double XIf() {return domain.XIf;}
	double Zsing() {return domain.Zsing;}
	double RHOsing() {return domain.RHOsing;}
	double ETAsing() {return domain.ETAsing;}
	double XIsing() {return domain.XIsing;}
	int levels() {return nlevels ;}
ifelse(NDIM,2,@
	void coarse_level(int nep, int nem, int ne3, int nsp, int nsm);
@,NDIM,3,@
	void coarse_level(int nep, int nem, int ne3, int nsp, int nsm, int nphi);
@)
	F_ID coarse_solve_ID(F_ID ,const int print = 0);
	void uniform_refine(const int);
	int adapt_refine(const int, const double);
	void initialize_level(const int lev, const double mask,
													const int Interp_Sol,
													const int Same_Uniform_Grid,
													cadez_discretization * Old_Level);
	void refine_solution(const int);
	void define_max_level(const int);
	void printlevels(ostream&);
	cadez_discretization & operator[](int);  // element selection
ifelse(NDIM,2,@
	void SetMomenta(const double Pzp, const double Pzm, const double Szp,
							const double Szm, const double Isom) {
		domain.Pzp = Pzp;
		domain.Pzm = Pzm;
		domain.Szp = Szp;
		domain.Szm = Szm;
		domain.Isom = Isom;
	}
@,NDIM,3,@
	void SetMomenta(const double Pxp, const double Pxm, const double Pyp,
							const double Pym, const double Pzp, const double Pzm,
							const double Sxp, const double Sxm, const double Syp,
							const double Sym, const double Szp, const double Szm,
							const double Isom) {
		domain.Pxp = Pxp;
		domain.Pxm = Pxm;
		domain.Pyp = Pyp;
		domain.Pym = Pym;
		domain.Pzp = Pzp;
		domain.Pzm = Pzm;
		domain.Sxp = Sxp;
		domain.Sxm = Sxm;
		domain.Syp = Syp;
		domain.Sym = Sym;
		domain.Szp = Szp;
		domain.Szm = Szm;
		domain.Isom = Isom;
	}
@)
	void New_Parameter_Set(const int);
	void mg_init_data(const int, F_ID&, F_ID&, const int pr = 0);
	friend ostream& operator<<(ostream& s, cadez_hierarchy& c) {
		s << c.domain;
		return s;
	}
};

#endif // CADEZ_HIERARCHY_H_
