#include "lparx.h"
ifelse(NDIM,2,@
#include "cadez_hierarchy2.h"
@,NDIM,3,@
#include "cadez_hierarchy3.h"
@)

ifelse(NDIM,2,@
#define I_X 0
#define I_Y 1
@,NDIM,3,@
#define I_W 0
#define I_X 1
#define I_Y 2
@)

ifelse(NDIM,2,@
extern "FORTRAN" double admsurfint(double *, double *, double *, double *,
			double *, double *, double *, int *, int *,
			const double &, const double &);
@,NDIM,3,@
extern "FORTRAN" double admsurfint(double *, double *, double *,
			const double &, double *, double *, double *, double *,
			int *, int *, const double &, const double &);
@)

extern "FORTRAN" double admvolint(double *, double *, double *, double *,
			int *, int *);

ifelse(NDIM,2,@
extern "FORTRAN" double zdipsurfint(double *, double *, double *, double *,
			double *, double *, double *, double *, int *, int *,
			const double &, const double &);
@,NDIM,3,@
extern "FORTRAN" double xdipsurfint(double *, double *, double *, double *,
			const double &,  double *, double *, double *, double *,
			double *, int *, int *, const double &, const double &);

extern "FORTRAN" double ydipsurfint(double *, double *, double *, double *,
			const double &,  double *, double *, double *, double *,
			double *, int *, int *, const double &, const double &);

extern "FORTRAN" double zdipsurfint(double *, double *, double *,
			const double &,  double *, double *, double *, double *,
			double *, int *, int *, const double &, const double &);
@)

ifelse(NDIM,3,@
extern "FORTRAN" double xdipvolint(double *, double *, double *, double *,
			double *, double *, int *, int *);

extern "FORTRAN" double ydipvolint(double *, double *, double *, double *,
			double *, double *, int *, int *);
@)
extern "FORTRAN" double zdipvolint(double *, double *, double *, double *,
			double *, int *, int *);

ifelse(NDIM,2,@
extern "FORTRAN" double surfarea(double *, double *, double *, double *,
			double *, double *, double *, int *, int *,
			const double &, const double &);
@,NDIM,3,@
extern "FORTRAN" double surfarea(double *, double *, double *,
			const double &, double *, double *, double *, double *,
			int *, int *, const double &, const double &);
@)

ifelse(NDIM,2,@
extern "FORTRAN" int propsep(double *, double *, const int &, double *,
			double *, double *, double *, double *, double *, int *, int *, int *,
			const double &, const double &, const double &, const double &);
@,NDIM,3,@
extern "FORTRAN" int propsep(double *, double *, int *, const int &, double *,
			double *, double *, double *, double *, double *, int *, int *, int *,
			const double &, const double &, const double &, const double &);
@)

extern "FORTRAN" void spline(double *, double *, const int &,
			const double &, const double &, double *);

extern "FORTRAN" double splint(double *, double *, double *, const int &);

double ADM_energy(cadez_hierarchy & cadez)
{
	const double xising = cadez.XIsing();
	const double alpha = cadez.Alpha();
	const int Nlevs = cadez.levels();
	double Energy = 0.0;
	for(int l=0; l < Nlevs; ++l) {
		cadez_discretization & lev = cadez[l];
		for_all_1(i,lev.Refine_Mask)
		//------------------------------------------------------------------------
		// Compute Surface Integral contribution.
		//
			if(lev.interiors[i].lower(I_Y) == lev.fills[i].lower(I_Y)) {
			// if on an inner Eta boundary then compute surface integral.
ifelse(NDIM,2,@
				Energy += admsurfint(lev.Psi(i).data(),
								lev.Refine_Mask(i).data(),lev.Xi(i).data(),
								lev.JXzc(i).data(),lev.JYzf(i).data(),
								lev.RHOyf(i).data(),lev.CJACyf(i).data(),
								(int *)(lev.extents+i),(int *)(lev.interiors+i),
								xising,alpha);
@,NDIM,3,@
				Energy += admsurfint(lev.Psi(i).data(),lev.Refine_Mask(i).data(),
								lev.Xi(i).data(),lev.dphi,
								lev.JXzc(i).data(),lev.JYzf(i).data(),
								lev.RHOyf(i).data(),lev.CJACyf(i).data(),
								(int *)(lev.extents+i),(int *)(lev.interiors+i),
								xising,alpha);
@)
			}
		//------------------------------------------------------------------------
		// Compute Volume Integral contribution.
		//
				Energy += admvolint(lev.Psi(i).data(),lev.KK(i).data(),
								lev.Refine_Mask(i).data(),lev.Volume(i).data(),
								(int *)(lev.extents+i),(int *)(lev.interiors+i));
		end_for_all_no_sync
	}
	mpReduceAdd(&Energy);
	return Energy;
} // ADM_energy

ifelse(NDIM,3,@
double Dipole_X(cadez_hierarchy & cadez)
//##############################################################################
//                           !!!!! WARNING !!!!!                              //
//        Computation of volume integral contribution to dipole moment        //
//        on adaptive grids will procuce poor results in the case when        //
//        the system has a net momentum at infinity.                          //
//##############################################################################
{
	const double xising = cadez.XIsing();
	const double alpha = cadez.Alpha();
	const int Nlevs = cadez.levels();
	double Dipole = 0.0;
	for(int l=0; l < Nlevs; ++l) {
		cadez_discretization & lev = cadez[l];
		for_all_1(i,lev.Refine_Mask)
		//------------------------------------------------------------------------
		// Compute Surface Integral contribution.
		//
			if(lev.interiors[i].lower(I_Y) == lev.fills[i].lower(I_Y)) {
			// if on an inner Eta boundary then compute surface integral.
				Dipole += xdipsurfint(lev.Psi(i).data(),lev.Refine_Mask(i).data(),
								lev.Phi(i).data(),lev.Xi(i).data(),lev.dphi,
								lev.JXzc(i).data(),lev.JYzf(i).data(),
								lev.RHOyf(i).data(),lev.Zyf(i).data(),
								lev.CJACyf(i).data(),(int *)(lev.extents+i),
								(int *)(lev.interiors+i),xising,alpha);
			}
		//------------------------------------------------------------------------
		// Compute Volume Integral contribution.
		//
				Dipole += xdipvolint(lev.Psi(i).data(),lev.KK(i).data(),
								lev.Refine_Mask(i).data(),lev.Phi(i).data(),
								lev.RHOzc(i).data(),lev.Volume(i).data(),
								(int *)(lev.extents+i),(int *)(lev.interiors+i));
		end_for_all_no_sync
	}
	mpReduceAdd(&Dipole);
	return Dipole;
} // Dipole_X
@)

ifelse(NDIM,3,@
double Dipole_Y(cadez_hierarchy & cadez)
//##############################################################################
//                           !!!!! WARNING !!!!!                              //
//        Computation of volume integral contribution to dipole moment        //
//        on adaptive grids will procuce poor results in the case when        //
//        the system has a net momentum at infinity.                          //
//##############################################################################
{
	const double xising = cadez.XIsing();
	const double alpha = cadez.Alpha();
	const int Nlevs = cadez.levels();
	double Dipole = 0.0;
	for(int l=0; l < Nlevs; ++l) {
		cadez_discretization & lev = cadez[l];
		for_all_1(i,lev.Refine_Mask)
		//------------------------------------------------------------------------
		// Compute Surface Integral contribution.
		//
			if(lev.interiors[i].lower(I_Y) == lev.fills[i].lower(I_Y)) {
			// if on an inner Eta boundary then compute surface integral.
				Dipole += ydipsurfint(lev.Psi(i).data(),lev.Refine_Mask(i).data(),
								lev.Phi(i).data(),lev.Xi(i).data(),lev.dphi,
								lev.JXzc(i).data(),lev.JYzf(i).data(),
								lev.RHOyf(i).data(),lev.Zyf(i).data(),
								lev.CJACyf(i).data(),(int *)(lev.extents+i),
								(int *)(lev.interiors+i),xising,alpha);
			}
		//------------------------------------------------------------------------
		// Compute Volume Integral contribution.
		//
				Dipole += ydipvolint(lev.Psi(i).data(),lev.KK(i).data(),
								lev.Refine_Mask(i).data(),lev.Phi(i).data(),
								lev.RHOzc(i).data(),lev.Volume(i).data(),
								(int *)(lev.extents+i),(int *)(lev.interiors+i));
		end_for_all_no_sync
	}
	mpReduceAdd(&Dipole);
	return Dipole;
} // Dipole_Y
@)

double Dipole_Z(cadez_hierarchy & cadez)
//##############################################################################
//                           !!!!! WARNING !!!!!                              //
//        Computation of volume integral contribution to dipole moment        //
//        on adaptive grids will procuce poor results in the case when        //
//        the system has a net momentum at infinity.                          //
//##############################################################################
{
	const double xising = cadez.XIsing();
	const double alpha = cadez.Alpha();
	const int Nlevs = cadez.levels();
	double Dipole = 0.0;
	for(int l=0; l < Nlevs; ++l) {
		cadez_discretization & lev = cadez[l];
		for_all_1(i,lev.Refine_Mask)
		//------------------------------------------------------------------------
		// Compute Surface Integral contribution.
		//
			if(lev.interiors[i].lower(I_Y) == lev.fills[i].lower(I_Y)) {
			// if on an inner Eta boundary then compute surface integral.
ifelse(NDIM,2,@
				Dipole += zdipsurfint(lev.Psi(i).data(),
								lev.Refine_Mask(i).data(),lev.Xi(i).data(),
								lev.JXzc(i).data(),lev.JYzf(i).data(),
								lev.RHOyf(i).data(),lev.Zyf(i).data(),
								lev.CJACyf(i).data(),(int *)(lev.extents+i),
								(int *)(lev.interiors+i),xising,alpha);
@,NDIM,3,@
				Dipole += zdipsurfint(lev.Psi(i).data(),lev.Refine_Mask(i).data(),
								lev.Xi(i).data(),lev.dphi,
								lev.JXzc(i).data(),lev.JYzf(i).data(),
								lev.RHOyf(i).data(),lev.Zyf(i).data(),
								lev.CJACyf(i).data(),(int *)(lev.extents+i),
								(int *)(lev.interiors+i),xising,alpha);
@)
			}
		//------------------------------------------------------------------------
		// Compute Volume Integral contribution.
		//
				Dipole += zdipvolint(lev.Psi(i).data(),lev.KK(i).data(),
								lev.Refine_Mask(i).data(),lev.Zzc(i).data(),
								lev.Volume(i).data(),
								(int *)(lev.extents+i),(int *)(lev.interiors+i));
		end_for_all_no_sync
	}
	mpReduceAdd(&Dipole);
	return Dipole;
} // Dipole_Z

double MS_Area(cadez_hierarchy & cadez, const int which_hole)
{
	const double xising = cadez.XIsing();
	const double alpha = cadez.Alpha();
	const int Nlevs = cadez.levels();
	double Area = 0.0;
	for(int l=0; l < Nlevs; ++l) {
		cadez_discretization & lev = cadez[l];
		RegionX holemask;
		if(which_hole > 0) holemask = lev.domain_R1;
		else if(which_hole < 0) holemask = lev.domain_R2;
		for_all_1(i,lev.Refine_Mask)
			if(lev.interiors[i].lower(I_Y) == lev.fills[i].lower(I_Y)) {
			// if on an inner Eta boundary then compute surface integral.
			//	Now check that the patch encompases the correct hole
				const RegionX ch = lev.interiors[i]*holemask;
				if(!ch.empty()) {
ifelse(NDIM,2,@
					Area += surfarea(lev.Psi(i).data(),
								lev.Refine_Mask(i).data(),lev.Xi(i).data(),
								lev.JXzc(i).data(),lev.JYzf(i).data(),
								lev.RHOyf(i).data(),lev.CJACyf(i).data(),
								(int *)(lev.extents+i),(int *)(lev.interiors+i),
								xising,alpha);
@,NDIM,3,@
					Area += surfarea(lev.Psi(i).data(),lev.Refine_Mask(i).data(),
								lev.Xi(i).data(),lev.dphi,
								lev.JXzc(i).data(),lev.JYzf(i).data(),
								lev.RHOyf(i).data(),lev.CJACyf(i).data(),
								(int *)(lev.extents+i),(int *)(lev.interiors+i),
								xising,alpha);
@)
				}
			}
		end_for_all_no_sync
	}
	mpReduceAdd(&Area);
	return Area;
} // MS_Area


#define PZListNULL ((PZList *) 0)
class PZList {
public:
	PZList *next;
	double psi, z;
	int count;
	PZList(double a, double b, int c = 1) {
		psi = a;
		z = b;
		count = c;
		next = PZListNULL;
	}
};

double MS_Separation(cadez_hierarchy & cadez)
{
	const int Id = mpMyID();
	const int dsize = sizeof(double);
	const int isize = sizeof(int);
	const double zetap = cadez.ZETAp();
	const double zetam = cadez.ZETAm();
	const double xising = cadez.XIsing();
	const double alpha = cadez.Alpha();
	const int Nlevs = cadez.levels();
	double *psi_vals;
	double *z_vals;
	ifelse(NDIM,3,int *count_vals;)
	double *psi2_vals;
	double Separation = 0.0;
//
//	Make sorted linked list of all psi and z values entering integral
	PZList *pl = PZListNULL;
	int PZcount = 0;
	for(int l=0; l < Nlevs; ++l) {
		cadez_discretization & lev = cadez[l];
		for_1(i,lev.Refine_Mask)
			const RegionX R12mask = lev.interiors[i]*
													(lev.domain_R1 + lev.domain_R2);
			if(!R12mask.empty()) {
				const int maxsize = lev.interiors[i].extents(I_Y) + 1;
				if(Id == 0) {
					const int Id_rcv = lev.mapping[i];
					psi_vals = new double[maxsize];
					z_vals = new double[maxsize];
					ifelse(NDIM,3,count_vals = new int[maxsize];)
					int nvals;
					if(Id_rcv == Id) {
					// compute psi and z values
ifelse(NDIM,2,@
						nvals = propsep(psi_vals,z_vals,maxsize,
									lev.Psi(i).data(),lev.Refine_Mask(i).data(),
									lev.Eta(i).data(),lev.Xi(i).data(),
									lev.JYzf(i).data(),lev.CJACyf(i).data(),
									(int *)(lev.extents+i),(int *)(lev.fills+i),
									(int *)(lev.interiors+i),zetap,zetam,
									xising,alpha);
@,NDIM,3,@
						nvals = propsep(psi_vals,z_vals,count_vals,maxsize,
									lev.Psi(i).data(),lev.Refine_Mask(i).data(),
									lev.Eta(i).data(),lev.Xi(i).data(),
									lev.JYzf(i).data(),lev.CJACyf(i).data(),
									(int *)(lev.extents+i),(int *)(lev.fills+i),
									(int *)(lev.interiors+i),zetap,zetam,
									xising,alpha);
@)

					} else {
					// receive psi and z values from another processor
						mpRecv(Id_rcv,&nvals,isize);
						mpRecv(Id_rcv,psi_vals,dsize*maxsize);
						ifelse(NDIM,3,mpRecv(Id_rcv,count_vals,isize*maxsize);)
						mpRecv(Id_rcv,z_vals,dsize*maxsize);
					}
				//	Sort psi and z values into linked list
               for(int j = 0; j < nvals; ++j ) {
						PZList **head = &pl;
ifelse(NDIM,2,@
						PZList *entry = new PZList(psi_vals[j],z_vals[j]);
@,NDIM,3,@
						PZList *entry = new PZList(psi_vals[j],z_vals[j],
																				count_vals[j]);
@)
						do {
							if((*head == PZListNULL) || (z_vals[j] < (*head)->z)) {
								entry->next = *head;
								*head = entry;
								++PZcount;
								break;
							} else if(z_vals[j] == (*head)->z) {
								(*head)->psi += entry->psi;
								(*head)->count += entry->count;
								delete entry;
								break;
							} else {
								head = &(*head)->next;
								continue;
							}
						}
						while(1);
					}
					delete psi_vals;
					delete z_vals;
					ifelse(NDIM,3,delete count_vals;)
				} else if(lev.mapping[i] == Id) {
					psi_vals = new double[maxsize];
					z_vals = new double[maxsize];
					ifelse(NDIM,3,count_vals = new int[maxsize];)
					int nvals;
				// compute psi and z values and sent to processor 0
ifelse(NDIM,2,@
					nvals = propsep(psi_vals,z_vals,maxsize,
								lev.Psi(i).data(),lev.Refine_Mask(i).data(),
								lev.Eta(i).data(),lev.Xi(i).data(),
								lev.JYzf(i).data(),lev.CJACyf(i).data(),
								(int *)(lev.extents+i),(int *)(lev.fills+i),
								(int *)(lev.interiors+i),zetap,zetam,
								xising,alpha);
@,NDIM,3,@
					nvals = propsep(psi_vals,z_vals,count_vals,maxsize,
								lev.Psi(i).data(),lev.Refine_Mask(i).data(),
								lev.Eta(i).data(),lev.Xi(i).data(),
								lev.JYzf(i).data(),lev.CJACyf(i).data(),
								(int *)(lev.extents+i),(int *)(lev.fills+i),
								(int *)(lev.interiors+i),zetap,zetam,
								xising,alpha);
@)
					mpSend(Id,&nvals,isize,0);
					mpSend(Id,psi_vals,dsize*nvals,0);
					ifelse(NDIM,3,mpSend(Id,count_vals,isize*nvals,0);)
					mpSend(Id,z_vals,dsize*nvals,0);
					delete psi_vals;
					delete z_vals;
					ifelse(NDIM,3,delete count_vals;)
				}
			}
		end_for
	}
	if(Id == 0) {
	//
	//	Turn Linked list into arrays
		psi_vals = new double[PZcount];
		z_vals = new double[PZcount];
		psi2_vals = new double[PZcount];
		for(int i = 0; i < PZcount; ++i) {
			psi_vals[i] = pl->psi/pl->count;
			z_vals[i] = pl->z;
			PZList *remove = pl;
			pl = pl->next;
			delete remove;
		}
	//
	//	Compute spline of data
	// NOTE!!! The call to spline here assumes INVERSION SYMMETRIC boundary
	//         condintions on the throats!!!
      spline(z_vals,psi_vals,PZcount,-0.5*alpha*(*psi_vals),
													0.5*(*(psi_vals+PZcount-1)),psi2_vals);
      Separation = splint(z_vals,psi_vals,psi2_vals,PZcount);
		mpBroadcast(&Separation,sizeof(Separation),0);
	} else {
		mpBroadcast(&Separation,sizeof(Separation),0);
	}
//
//	Compute separation using integral of cubic spline
	return Separation;
} // MS_Separation
